@@ -373,6 +373,8 @@ class RandomCompositionMutation(SlabOperator):
                 indi = tmp.copy()
                 indi.info = info
 
+        if 'groups' in f.info['data']:
+            indi.info['data']['groups'] = f.info['data']['groups']
         indi.info['data']['parents'] = [i.info['confid'] for i in parents]
 
         return (self.finalize_individual(indi),
@@ -435,7 +437,12 @@ class RandomSlabPermutation(SlabOperator):
             f = f0
         else:
             if any(isinstance(i, list) for i in self.allowed_indices):
-                self.allowed_indices = random.choice(self.allowed_indices)
+                allowed_indices = [idxs for idxs in self.allowed_indices if 
+                                   len(set(f0[idxs].get_chemical_symbols())) > 1]
+                if not allowed_indices:
+                    return None, '{1} not possible in {0}'.format(f0.info['confid'],
+                                                                  self.descriptor)
+                self.allowed_indices = random.choice(allowed_indices)
             f = f0[self.allowed_indices]
 
         # Permutation only makes sense if two different elements are present
@@ -463,6 +470,8 @@ class RandomSlabPermutation(SlabOperator):
                 indi = tmp.copy()
                 indi.info = info
 
+        if 'groups' in f.info['data']:
+            indi.info['data']['groups'] = f.info['data']['groups']
         indi.info['data']['parents'] = [i.info['confid'] for i in parents]
         parent_message = ': Parent {0}'.format(f.info['confid'])
 
